/*
 * @Author: zhanyoulin<zhanyoulin456@163.com>
 * @Date: 2023-06-25 09:01:18
 * @LastEditors: xiejiang
 * @LastEditTime: 2024-12-10 18:00:50
 * @Description:
 */
import axios, { AxiosInstance, AxiosError, AxiosRequestConfig, InternalAxiosRequestConfig, AxiosResponse } from 'axios'
import qs from 'qs'
import { tryHideFullScreenLoading } from '@/config/serviceLoading'
import { LOGIN_URL } from '@/config'
import { MessagePlugin } from 'tdesign-vue-next'
import { ResultData } from '@/api/interface'
import { ResultEnum } from '@/enums/httpEnum'
import { checkStatus } from './helper/checkStatus'
import { useUserStore } from '@/stores/modules/user'
import { useAuthStore } from '@/stores/modules/auth'
import { getDeviceType } from '@/utils/device'
import router from '@/routers'

export interface CustomAxiosRequestConfig extends InternalAxiosRequestConfig {
  noToken?: boolean
  noLoading?: boolean
}

const config = {
  // 默认地址请求地址，可在 .env.** 文件中修改
  baseURL: import.meta.env.PUBLIC_API_URL as string,
  // 设置超时时间
  timeout: ResultEnum.TIMEOUT as number,
  paramsSerializer: function (params: any) {
    params = params || {}
    return qs.stringify(params)
  },
  // 跨域时候允许携带凭证
  withCredentials: false
}

class RequestHttp {
  service: AxiosInstance

  /**
   * @description 显示错误消息的方法
   * @param msg 错误消息
   */
  private showMessage(msg: string) {
    if (!this.isMessageShown) {
      this.isMessageShown = true
      MessagePlugin.error(msg)
      setTimeout(() => {
        this.isMessageShown = false
      }, 2000)
    }
  }
  private isMessageShown: boolean = false // 添加一个标志来跟踪是否已经显示了消息
  public constructor(config: AxiosRequestConfig) {
    // instantiation
    this.service = axios.create(config)

    /**
     * @description 请求拦截器
     * 客户端发送请求 -> [请求拦截器] -> 服务器
     * token校验: 接受服务器返回的 token,存储到 vuex/pinia/本地储存当中
     */
    this.service.interceptors.request.use(
      (config: CustomAxiosRequestConfig) => {
        const userStore = useUserStore()
        const authStore = useAuthStore()
        // 当前请求不需要显示 loading，在 api 服务中通过指定的第三个参数: { noLoading: true } 来控制
        // config.noLoading || showFullScreenLoading()
        if (config.headers && typeof config.headers.set === 'function') {
          const activeMenu = authStore.activeMenuId || localStorage.getItem('activeMenuId')
          config.headers.set('menuId', activeMenu)

          // config.headers.set('menuSign', authStore.menuSign)
          config.headers.set('client', 'platform')
          config.headers.set('device', getDeviceType())
          config.headers.set('ngrok-skip-browser-warning', 69420)
          if (!config.noToken) {
            config.headers.set('Authorization', `Bearer ${userStore.token}`)
          }
        }
        return config
      },
      (error: AxiosError) => {
        return Promise.reject(error)
      }
    )

    /**
     * @description 响应拦截器
     *  服务器换返回信息 -> [拦截统一处理] -> 客户端JS获取到信息
     */
    this.service.interceptors.response.use(
      (response: AxiosResponse) => {
        const { data } = response
        const userStore = useUserStore()
        tryHideFullScreenLoading()
        // 登陆失效
        if (data.code == ResultEnum.OVERDUE) {
          userStore.setToken('')
          router.replace(LOGIN_URL)
          this.showMessage(data.msg)
          return Promise.reject(data)
        }
        // 全局错误信息拦截（防止下载文件的时候返回数据流，没有 code 直接报错）
        if (data.code && data.code !== ResultEnum.SUCCESS) {
          this.showMessage(data.msg)
          return Promise.reject(data)
        }
        // 成功请求（在页面上除非特殊情况，否则不用处理失败逻辑）
        return data
      },
      async (error: AxiosError) => {
        const { response } = error
        tryHideFullScreenLoading()
        // 请求超时 && 网络错误单独判断，没有 response
        if (error.message.indexOf('timeout') !== -1) this.showMessage('请求超时！请您稍后重试')
        if (error.message.indexOf('Network Error') !== -1) this.showMessage('网络错误！请您稍后重试')
        // 根据服务器响应的错误状态码，做不同的处理
        if (response) checkStatus(response.status)
        // 服务器结果都没有返回(可能服务器错误可能客户端断网)，断网处理:可以跳转到断网页面
        if (!window.navigator.onLine) router.replace('/500')
        return Promise.reject(error)
      }
    )
  }

  /**
   * @description 常用请求方法封装
   */
  get<T>(url: string, params?: object, _object = {}): Promise<ResultData<T>> {
    return this.service.get(url, { params, ..._object })
  }
  post<T>(url: string, params?: object | string, _object = {}): Promise<ResultData<T>> {
    return this.service.post(url, params, _object)
  }
  put<T>(url: string, params?: object, _object = {}): Promise<ResultData<T>> {
    return this.service.put(url, params, _object)
  }
  delete<T>(url: string, params?: any, _object = {}): Promise<ResultData<T>> {
    return this.service.delete(url, { params, ..._object })
  }
  download(url: string, params?: object, _object = {}): Promise<BlobPart> {
    return this.service.post(url, params, { ..._object, responseType: 'blob' })
  }
}

export default new RequestHttp(config)
